AGN_CONFIG = {};
AGN_CONFIG.debug = false;
AGN_CONFIG.enabled = true;
AGN_CONFIG.enableAlts = false;
--- An array of field IDs
--- @type table<number, string>
AGN_CONFIG.enabledFieldIds = {};

--- @return number startIdx
--- @return string config
--- @return number endIdx
function AGN:GetConfigFromGuildInfo()
    --- @type string
    local text = self.player.guild.info;

    return text:match("()AGN%^(.-)%^()");
end

--- @return string
function AGN:ConfigToGuildInfoFormat()
    local config = "AGN^";
    -- LuaFormatter off
    local enabledFields = F.Map(AGN_CONFIG.enabledFieldIds,
        function(fieldId)
            local field = F.FindFirst(AGN_DATA.fields,
                --- @param f FieldInfo
                function(f) return f.id == fieldId; end);
            return field.guildInfoId;
        end);
    -- LuaFormatter on
    config = config .. enabledFields;
    config = config .. ":";
    config = config .. (AGN_CONFIG.enableAlts and "1" or "0");
    config = config .. "^";

    return config;
end

function AGN:TryUpdateConfigFromGuildInfo()
    Log.d("Updating config from guild info");
    local startIdx, config = self:GetConfigFromGuildInfo();
    Log.d("Got config from guild info", config);
    if config == nil then return; end

    Log.d("Found config", config, "at position", startIdx);

    --- @type string
    local enabledFields, restConfig = strsplit(":", config);
    Log.d("Enabling fields", enabledFields);

    AGN_CONFIG.enabledFieldIds = {};
    for i = 1, #enabledFields do
        local fieldGuildInfoId = enabledFields:sub(i, i);
        -- LuaFormatter off
        local field = F.FindFirst(AGN_DATA.fields,
            --- @param f FieldInfo
            function(f)
                return f.guildInfoId == fieldGuildInfoId;
            end);
        -- LuaFormatter on
        self:EnableField(field);
    end

    restConfig = restConfig or "0";
    Log.d("Rest config", restConfig);
    AGN_CONFIG.enableAlts = restConfig:sub(1, 1) == "1";
end

function AGN:TryWriteConfigToGuildInfo()
    if CanEditGuildInfo() == false then
        Log.e("You don't have permission to edit the guild info");
        return;
    end

    Log.d("Writing config to guild info");
    local startIdx, config, endIdx = self:GetConfigFromGuildInfo();
    --- @type string
    local guildInfo = GetGuildInfoText();
    local newConfig = self:ConfigToGuildInfoFormat();
    Log.d("New config", newConfig);
    local newGuildInfo;
    if config ~= nil then
        Log.d("Overwriting old config at position", startIdx);
        newGuildInfo = guildInfo:sub(1, startIdx - 1) .. newConfig .. guildInfo:sub(endIdx);
    else
        newGuildInfo = guildInfo .. "\n" .. newConfig;
    end
    Log.d("New guild info", newGuildInfo);
    SetGuildInfoText(newGuildInfo);
end

--- @param field FieldInfo
function AGN:EnableField(field)
    if field == nil then return; end
    if tContains(AGN_CONFIG.enabledFieldIds, field.id) then return; end

    Log.d("Enabling field" .. field.id);
    table.insert(AGN_CONFIG.enabledFieldIds, field.id);
end

--- @param field FieldInfo
function AGN:DisableField(field)
    if field == nil then return; end

    Log.d("Disabling field" .. field.id);
    -- LuaFormatter off
    AGN_CONFIG.enabledFieldIds = F.Filter(AGN_CONFIG.enabledFieldIds,
        --- @param fieldId string
        function (fieldId)
            return fieldId ~= field.id;
        end);
    -- LuaFormatter on
end

--- @param field FieldInfo
--- @return boolean
function AGN:ToggleFieldEnabled(field)
    if field == nil then return; end

    local isEnabled = self:IsFieldEnabled(field);
    if isEnabled then
        self:DisableField(field)
    else
        self:EnableField(field)
    end

    return not isEnabled;
end

--- @param field FieldInfo
--- @return boolean
function AGN:IsFieldEnabled(field)
    -- LuaFormatter off
    return F.Any(AGN_CONFIG.enabledFieldIds,
        --- @param fieldId string
        function(fieldId)
            return fieldId == field.id;
        end);
    -- LuaFormatter on
end
