function AGN.frame:OnEvent(event, ...)
    local handler = self[event];
    if (handler == nil) then return; end

    handler(self, event, ...);
end

function AGN.frame:ADDON_LOADED(_, addonName)
    if addonName ~= "AutoGuildNote" then return; end

    AGN_CONFIG.debug = (AGN_CONFIG.debug ~= nil) and AGN_CONFIG.debug or false;
    AGN_CONFIG.enabled = (AGN_CONFIG.enabled ~= nil) and AGN_CONFIG.enabled or true;
    AGN_CONFIG.enableAlts = (AGN_CONFIG.enableAlts ~= nil) and AGN_CONFIG.enableAlts or false;
    AGN_CONFIG.enabledFieldIds = AGN_CONFIG.enabledFieldIds or {};
end

function AGN.frame:PLAYER_LOGIN(_)
    self:RegisterEvent("GUILD_ROSTER_UPDATE");

    local playerName, playerRealm = UnitFullName("player");
    AGN.player.name = playerName;
    AGN.player.realm = playerRealm;
    AGN.player.fullName = playerName .. "-" .. playerRealm;
    AGN.player.guid = UnitGUID("player");
    _, AGN.player.race = UnitRace("player");

    -- local guildName, guildRankName, guildRankIndex, guildRealm =
    --     GetGuildInfo("player");
    -- AGN.player.guild.name = guildName;
    -- AGN.player.guild.realm = guildRealm;
    -- AGN.player.guild.fullName = guildName .. "-" .. guildRealm;
    -- AGN.player.guild.rankIndex = guildRankIndex;
    -- AGN.player.guild.rankName = guildRankName;

    Log.d("AGN loaded");
    Log.d("Player GUID", AGN.player.guid);
    Log.d("Player full name", AGN.player.fullName);

    C_GuildInfo.GuildRoster();
    AGN:TryUpdateNote();
end

function AGN.frame:GUILD_ROSTER_UPDATE(_)
    AGN.player.guild.rankOrder = C_GuildInfo.GetGuildRankOrder(AGN.player.guid);
    AGN.player.guild.rankName = GuildControlGetRankName(AGN.player.guild.rankOrder);
    AGN.player.guild.info = GetGuildInfoText();

    AGN:TryUpdateConfigFromGuildInfo();
    AGN:TryUpdateNote();
end

function AGN.frame:INSPECT_READY(_, targetGuid)
    if not AGN_CONFIG.enabled then return; end
    if not AGN:IsReady() then
        Log.d("Not ready to update note");
        return;
    end

    -- don't set note for alt ranks
    if (not AGN_CONFIG.enableAlts and string.find(string.lower(AGN.player.guild.rankName), "alt") ~= nil) then
        Log.d("Not setting guild note for rank", AGN.player.guild.rankName);
        self:UnregisterEvent("GUILD_ROSTER_UPDATE");
        self:UnregisterEvent("INSPECT_READY");
        return;
    end

    if (targetGuid ~= AGN.player.guid) then return; end

    local fieldValues = {};
    for _, fieldId in pairs(AGN_CONFIG.enabledFieldIds) do
        -- LuaFormatter off
        local field = F.FindFirst(AGN_DATA.fields,
            --- @param value FieldInfo
            function(value)
                return value.id == fieldId;
            end);
        -- LuaFormatter on
        local value = field:getString();

        if value ~= nil then table.insert(fieldValues, value); end
    end

    local note = strjoin("/", unpack(fieldValues));

    Log.i("Setting own public note to", note);

    for i = 1, GetNumGuildMembers() do
        if GetGuildRosterInfo(i) == AGN.player.fullName then
            GuildRosterSetPublicNote(i, note);
            self:UnregisterEvent("GUILD_ROSTER_UPDATE");
            self:UnregisterEvent("INSPECT_READY");
            break
        end
    end

    ClearInspectPlayer();
end

AGN.frame:RegisterEvent("ADDON_LOADED");
AGN.frame:RegisterEvent("PLAYER_LOGIN");
AGN.frame:SetScript("OnEvent", AGN.frame.OnEvent);
