F = {};

--- @generic TKey, TValue, TResult
--- @param iterable table<TKey, TValue>
--- @param accumulator fun(intermediaryResult: TResult, value: TValue, key: TKey): TResult
--- @param seed TResult The initial value
--- @return TResult
function F.Fold(iterable, accumulator, seed)
    local result = seed;
    for key, value in pairs(iterable) do result = accumulator(result, value, key); end

    return result;
end

--- @generic TKey, TValue
--- @param iterable table<TKey, TValue>
--- @param predicate fun(value: TValue, key: TKey): boolean
--- @return TValue
--- @return TKey
function F.FindFirst(iterable, predicate)
    local key, value = next(F.Filter(iterable, predicate));
    return value, key;
end

--- @generic TKey, TValue, TResult
--- @param iterable table<TKey, TValue>
--- @param mapper fun(value: TValue, key: TKey): TResult
--- @return table<TKey, TResult>
function F.Map(iterable, mapper)
    return F.Fold(iterable, function(result, ...)
        table.insert(result, mapper(...));
        return result;
    end, {})
end

--- @generic TKey, TValue
--- @param iterable table<TKey, TValue>
--- @param predicate fun(value: TValue, key: TKey): boolean
--- @return table<TKey, TValue>
function F.Filter(iterable, predicate)
    return F.Fold(iterable, function(result, value, ...)
        if predicate(value, ...) then table.insert(result, value) end
        return result;
    end, {})
end

--- @generic TKey, TValue
--- @param iterable table<TKey, TValue>
--- @param predicate fun(value: TValue, key: TKey): boolean
--- @return boolean
function F.Any(iterable, predicate)
    return F.Fold(iterable, function(result, value, ...)
        if predicate(value, ...) then return true; end
        return result;
    end, false);
end

function F.All(iterable, predicate)
    return F.Fold(iterable, function(result, value, ...)
        if not predicate(value) then return false; end
        return result;
    end, true);
end
