SLASH_AGN1 = "/agn";

--- @param msg string
--- @param editBox any
SlashCmdList.AGN = function(msg, editBox)
    Log.d("Called slash command", msg);
    local args = {};
    --- @type string
    local cmdString, rest = strsplit(" ", msg, 2);
    Log.d("Command", cmdString);
    if rest ~= nil then
        for arg in rest:gmatch("%a+") do
            Log.d("Command arg", arg);
            table.insert(args, arg);
        end
    end

    local cmd = AGN.commands[cmdString];
    if cmd ~= nil then
        cmd(unpack(args));
    else
        AGN.commands.help();
    end
end;

function AGN.commands.help()
    print([[AGN commands:
    debug - enables/disables debug messages
    enable - enables automatically setting the guild note
    disable - disables automatically setting the guild note
    writeconfig - writes the config to the guild note
    fields - lists all available fields and their status
    fields toggle <field> - toggles the field identified by <field>
    enablealts - Enable setting note for alts
    disablealts - Disable setting note for alts]]);
end

function AGN.commands.debug()
    AGN_CONFIG.debug = not AGN_CONFIG.debug;
    Log.i("AutoGuildNote Debugging", AGN_CONFIG.debug and "enabled" or "disabled");
end

function AGN.commands.enable()
    Log.i("Enabling AutoGuildNote");
    AGN_CONFIG.enabled = true;
end

function AGN.commands.disable()
    Log.i("Disabling AutoGuildNote");
    AGN_CONFIG.enabled = false;
end

function AGN.commands.writeconfig()
    Log.i("Trying to write config to guild info");
    AGN:TryWriteConfigToGuildInfo();
end

function AGN.commands.fields(command, fieldId)
    Log.d("Field command", command);
    Log.d("Field id", fieldId);
    if command == nil then
        --- @type FieldInfo
        for _, field in pairs(AGN_DATA.fields) do
            local isEnabled = AGN:IsFieldEnabled(field);
            Log.i(strjoin(" - ", field.id, field.description, isEnabled and "enabled" or "disabled"));
        end
        return;
    end

    if command == "toggle" then
        if fieldId == nil then
            Log.e("Must specify a field to toggle");
            return;
        end

        -- LuaFormatter off
        local field = F.FindFirst(AGN_DATA.fields,
            --- @param f FieldInfo
            function(f)
                return f.id == fieldId;
            end);
        -- LuaFormatter on
        if field == nil then
            Log.e("Invalid field", fieldId);
            return;
        end

        local isEnabled = AGN:ToggleFieldEnabled(field);
        Log.i(isEnabled and "Enabled" or "Disabled", "field", field.id);

        return;
    end
end

function AGN.commands.enablealts()
    Log.i("Enabling AutoGuildNote for alts");
    AGN_CONFIG.enableAlts = true;
end

function AGN.commands.disablealts()
    Log.i("Disabling AutoGuildNote for alts");
    AGN_CONFIG.enableAlts = false;
end
